// Inspired by
// https://observablehq.com/@d3/parallel-coordinates
// https://observablehq.com/@d3/brushable-parallel-coordinates

import {
  select,
  transition,
  scaleLinear,
  scalePoint,
  scaleOrdinal,
  schemeCategory10,
  extent,
  line,
  brushY,
} from 'd3';
import { data } from '@Ljz2018/7daysearthquakedata';
import { observeResize } from '@curran/responsive-axes';
import { parallelCoordinates } from './parallelCoordinates';

// Assign d.id to the index.
// This is used later for object constancy in transitions.
data.forEach((d, i) => {
  d.id = i;
});

//target columns:
//   "depth": 432.242,
//   "mag": 5.3,
//   "magType": "mww",
//   "nst": 83,
//   "gap": 59,
//   "dmin": 2.933,
//   "rms": 0.98,
//   "net": "us",
//   "horizontalError": 11.47,
//   "depthError": 6.669,
//   "magError": 0.089,
//   "magNst": 12,
//   "locationSource": "us",
//   "magSource": "us",
const columns = [
  'depth',
  'mag',
  'magType',
  'depthError',
  'dmin',
  'magError',
  'magNst',
  //'gap',
  'nst',
  //'rms',
  'horizontalError',
  //'locationSource',
  //'magSource',
  //'net',
];
const columnTypes = {
  depth: 'quantitative',
  mag: 'quantitative',
  magType: 'categorical',
  nst: 'quantitative',
  //gap: 'quantitative',
  dmin: 'quantitative',
  //rms: 'quantitative',
  horizontalError: 'quantitative',
  depthError: 'quantitative',
  magError: 'quantitative',
  magNst: 'quantitative',
  //locationSource: 'categorical',
  //magSource: 'categorical',
  //net: 'categorical',
};

// Categorize the depth values
function categorizeDepth(depth) {
  if (depth < 70) {
    return 'shallow: depth < 70km';
  } else if (depth < 300) {
    return 'intermediate: 70km ≤ depth < 300km';
  } else {
    return 'deep: depth ≥ 300km';
  }
}
const colorValue = (d) => categorizeDepth(d.depth);
const colorScale = scaleOrdinal()
  .domain(data.map(colorValue))
  .range(['red', 'blue', 'green']);
//const colorValue = (d) => d.depth;
const idValue = (d) => d.id;

export const main = (container, { state, setState }) => {
  // Initialize dimensions by observing resise.
  const dimensions = observeResize({
    state,
    setState,
    container,
  });
  if (dimensions === null) return;
  const { width, height } = dimensions;

  // Initialize the brushed intervals state to an empty object.
  if (!state.brushedIntervals) {
    setState((state) => ({
      ...state,
      brushedIntervals: {},
    }));
    return;
  }
  const { brushedIntervals } = state;

  // Updates the brushed intervals when brushes change.
  const updateBrushedInterval = ({ column, interval }) => {
    setState((state) => ({
      ...state,
      brushedIntervals: {
        ...state.brushedIntervals,
        [column]: interval,
      },
    }));
  };

  // Use this to inspect the brushedIntervals as they change!
  // console.log(
  //   JSON.stringify(brushedIntervals, null, 2),
  // );

  // Set up the SVG.
  const svg = select(container)
    .selectAll('svg')
    .data([null])
    .join('svg')
    .attr('width', width)
    .attr('height', height)
    .style('background', '#090F10');

  // Invoke the parallel coordinates.
  svg.call(parallelCoordinates, {
    data,
    columns,
    columnTypes,
    colorValue,
    idValue,
    width,
    height,
    brushedIntervals,
    updateBrushedInterval,
  });